"""Added ExternalSoftwareEnvironment, ExternalHardwareInformation, constraints for syncs

Revision ID: 9a8ce1e52efb
Revises: f60593780969
Create Date: 2019-10-07 07:40:32.070193+00:00

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = '9a8ce1e52efb'
down_revision = 'f60593780969'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('external_hardware_information',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('cpu_vendor', sa.Integer(), nullable=True),
    sa.Column('cpu_model', sa.Integer(), nullable=True),
    sa.Column('cpu_cores', sa.Integer(), nullable=True),
    sa.Column('cpu_model_name', sa.String(length=256), nullable=True),
    sa.Column('cpu_family', sa.Integer(), nullable=True),
    sa.Column('cpu_physical_cpus', sa.Integer(), nullable=True),
    sa.Column('gpu_model_name', sa.String(length=256), nullable=True),
    sa.Column('gpu_vendor', sa.String(length=256), nullable=True),
    sa.Column('gpu_cores', sa.Integer(), nullable=True),
    sa.Column('gpu_memory_size', sa.Integer(), nullable=True),
    sa.Column('ram_size', sa.Integer(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('external_software_environment',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('environment_name', sa.String(length=256), nullable=True),
    sa.Column('python_version', sa.String(length=256), nullable=True),
    sa.Column('image_name', sa.String(length=256), nullable=True),
    sa.Column('image_sha', sa.String(length=256), nullable=True),
    sa.Column('os_name', sa.String(length=256), nullable=True),
    sa.Column('os_version', sa.String(length=256), nullable=True),
    sa.Column('cuda_version', sa.String(length=256), nullable=True),
    sa.Column('environment_type', postgresql.ENUM('RUNTIME', 'BUILDTIME', name='environment_type'), nullable=False),
    sa.PrimaryKeyConstraint('id')
    )
    op.add_column('adviser_run', sa.Column('external_build_software_environment_id', sa.Integer(), nullable=True))
    op.add_column('adviser_run', sa.Column('external_hardware_information_id', sa.Integer(), nullable=True))
    op.add_column('adviser_run', sa.Column('external_run_software_environment_id', sa.Integer(), nullable=True))
    op.drop_constraint('adviser_run_user_build_software_environment_id_fkey', 'adviser_run', type_='foreignkey')
    op.drop_constraint('adviser_run_user_run_software_environment_id_fkey', 'adviser_run', type_='foreignkey')
    op.drop_constraint('adviser_run_hardware_information_id_fkey', 'adviser_run', type_='foreignkey')
    op.create_foreign_key(None, 'adviser_run', 'external_software_environment', ['external_run_software_environment_id'], ['id'], ondelete='CASCADE')
    op.create_foreign_key(None, 'adviser_run', 'external_software_environment', ['external_build_software_environment_id'], ['id'], ondelete='CASCADE')
    op.create_foreign_key(None, 'adviser_run', 'external_hardware_information', ['external_hardware_information_id'], ['id'], ondelete='CASCADE')
    op.drop_column('adviser_run', 'user_build_software_environment_id')
    op.drop_column('adviser_run', 'user_run_software_environment_id')
    op.drop_column('adviser_run', 'hardware_information_id')
    op.drop_column('hardware_information', 'is_user')
    op.alter_column('python_package_version', 'os_name',
               existing_type=sa.VARCHAR(length=256),
               nullable=False)
    op.alter_column('python_package_version', 'os_version',
               existing_type=sa.VARCHAR(length=256),
               nullable=False)
    op.alter_column('python_package_version', 'python_version',
               existing_type=sa.VARCHAR(length=256),
               nullable=False)
    op.drop_column('software_environment', 'is_user')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('software_environment', sa.Column('is_user', sa.BOOLEAN(), autoincrement=False, nullable=False))
    op.alter_column('python_package_version', 'python_version',
               existing_type=sa.VARCHAR(length=256),
               nullable=True)
    op.alter_column('python_package_version', 'os_version',
               existing_type=sa.VARCHAR(length=256),
               nullable=True)
    op.alter_column('python_package_version', 'os_name',
               existing_type=sa.VARCHAR(length=256),
               nullable=True)
    op.add_column('hardware_information', sa.Column('is_user', sa.BOOLEAN(), autoincrement=False, nullable=False))
    op.add_column('adviser_run', sa.Column('hardware_information_id', sa.INTEGER(), autoincrement=False, nullable=True))
    op.add_column('adviser_run', sa.Column('user_run_software_environment_id', sa.INTEGER(), autoincrement=False, nullable=True))
    op.add_column('adviser_run', sa.Column('user_build_software_environment_id', sa.INTEGER(), autoincrement=False, nullable=True))
    op.drop_constraint(None, 'adviser_run', type_='foreignkey')
    op.drop_constraint(None, 'adviser_run', type_='foreignkey')
    op.drop_constraint(None, 'adviser_run', type_='foreignkey')
    op.create_foreign_key('adviser_run_hardware_information_id_fkey', 'adviser_run', 'hardware_information', ['hardware_information_id'], ['id'], ondelete='CASCADE')
    op.create_foreign_key('adviser_run_user_run_software_environment_id_fkey', 'adviser_run', 'software_environment', ['user_run_software_environment_id'], ['id'], ondelete='CASCADE')
    op.create_foreign_key('adviser_run_user_build_software_environment_id_fkey', 'adviser_run', 'software_environment', ['user_build_software_environment_id'], ['id'], ondelete='CASCADE')
    op.drop_column('adviser_run', 'external_run_software_environment_id')
    op.drop_column('adviser_run', 'external_hardware_information_id')
    op.drop_column('adviser_run', 'external_build_software_environment_id')
    op.drop_table('external_software_environment')
    op.drop_table('external_hardware_information')
    # ### end Alembic commands ###



"""Recreate indexes for Python package version

Revision ID: b91c11e75551
Revises: 1fbd52a03c73
Create Date: 2020-01-13 21:37:05.476084+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'b91c11e75551'
down_revision = '1fbd52a03c73'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index('python_package_version_index_idx_00', 'python_package_version', ['package_name', 'package_version'], unique=False)
    op.drop_index('python_package_version_index_idx_10', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_10', 'python_package_version', ['package_name', 'package_version', 'os_name'], unique=False)
    op.drop_index('python_package_version_index_idx_11', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_11', 'python_package_version', ['package_name', 'package_version', 'os_version'], unique=False)
    op.drop_index('python_package_version_index_idx_12', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_12', 'python_package_version', ['package_name', 'package_version', 'python_version'], unique=False)
    op.drop_index('python_package_version_index_idx_20', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_20', 'python_package_version', ['package_name', 'package_version', 'os_name', 'os_version'], unique=False)
    op.drop_index('python_package_version_index_idx_21', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_21', 'python_package_version', ['package_name', 'package_version', 'os_name', 'python_version'], unique=False)
    op.drop_index('python_package_version_index_idx_22', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_22', 'python_package_version', ['package_name', 'package_version', 'os_version', 'python_version'], unique=False)
    op.drop_index('python_package_version_index_idx_30', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_30', 'python_package_version', ['package_name', 'package_version', 'os_name', 'os_version', 'python_version'], unique=False)
    op.drop_index('python_package_version_idx', table_name='python_package_version')
    op.drop_index('python_package_version_index_idx_13', table_name='python_package_version')
    op.drop_index('python_package_version_index_idx_23', table_name='python_package_version')
    op.drop_index('python_package_version_index_idx_24', table_name='python_package_version')
    op.drop_index('python_package_version_index_idx_25', table_name='python_package_version')
    op.drop_index('python_package_version_index_idx_31', table_name='python_package_version')
    op.drop_index('python_package_version_index_idx_32', table_name='python_package_version')
    op.drop_index('python_package_version_index_idx_33', table_name='python_package_version')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index('python_package_version_index_idx_33', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'os_name', 'os_version', 'python_version'], unique=False)
    op.create_index('python_package_version_index_idx_32', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'python_package_index_id', 'os_version', 'python_version'], unique=False)
    op.create_index('python_package_version_index_idx_31', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'python_package_index_id', 'os_name', 'python_version'], unique=False)
    op.create_index('python_package_version_index_idx_25', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'os_version', 'python_version'], unique=False)
    op.create_index('python_package_version_index_idx_24', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'os_name', 'python_version'], unique=False)
    op.create_index('python_package_version_index_idx_23', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'os_name', 'os_version'], unique=False)
    op.create_index('python_package_version_index_idx_13', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'python_version'], unique=False)
    op.create_index('python_package_version_idx', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id'], unique=False)
    op.drop_index('python_package_version_index_idx_30', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_30', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'python_package_index_id', 'os_name', 'os_version'], unique=False)
    op.drop_index('python_package_version_index_idx_22', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_22', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'python_package_index_id', 'python_version'], unique=False)
    op.drop_index('python_package_version_index_idx_21', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_21', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'python_package_index_id', 'os_version'], unique=False)
    op.drop_index('python_package_version_index_idx_20', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_20', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'python_package_index_id', 'os_name'], unique=False)
    op.drop_index('python_package_version_index_idx_12', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_12', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'os_version'], unique=False)
    op.drop_index('python_package_version_index_idx_11', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_11', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'os_name'], unique=False)
    op.drop_index('python_package_version_index_idx_10', table_name='python_package_version')
    op.create_index('python_package_version_index_idx_10', 'python_package_version', ['package_name', 'package_version', 'python_package_index_id', 'python_package_index_id'], unique=False)
    op.drop_index('python_package_version_index_idx_00', table_name='python_package_version')
    # ### end Alembic commands ###



#!/usr/bin/env python3
# thoth-storages
# Copyright(C) 2019, 2020 Fridolin Pokorny
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""A CLI client to thoth-storages library."""

import logging

import click
import daiquiri

from thoth.storages import __version__ as thoth_storages_version

daiquiri.setup(level=logging.INFO)
_LOGGER = logging.getLogger("thoth.storages")


def _print_version(ctx, _, value):
    """Print thoth-storages version and exit."""
    if not value or ctx.resilient_parsing:
        return

    click.echo(thoth_storages_version)
    ctx.exit()


@click.group()
@click.pass_context
@click.option(
    "-v", "--verbose", is_flag=True, envvar="THOTH_STORAGES_VERBOSE", help="Be verbose about what's going on."
)
@click.option(
    "--version",
    is_flag=True,
    is_eager=True,
    callback=_print_version,
    expose_value=False,
    help="Print version and exit.",
)
def cli(ctx=None, verbose: bool = False):
    """CLI tool for interacting with Thoth."""
    if ctx:
        ctx.auto_envvar_prefix = "THOTH_STORAGES"

    if verbose:
        _LOGGER.setLevel(logging.DEBUG)
        _LOGGER.debug("Debug mode is on")


@cli.command("generate-schema")
@click.argument("schema_file", type=str, required=False, default="schema.png", metavar="schema.png")
def generate_schema(schema_file: str):
    """Generate an image out of the current schema."""
    try:
        import sadisplay
        import pydot
    except ImportError:
        _LOGGER.error("Failed to import required libraries to perform schema generation")
        raise

    import thoth.storages.graph.models as models
    import thoth.storages.graph.models_performance as performance_models

    desc = sadisplay.describe(
        list(vars(models).values()) + list(vars(performance_models).values()),
        show_methods=True,
        show_properties=True,
        show_indexes=False,
        show_simple_indexes=False,
        show_columns_of_indexes=False,
    )
    dot_data = sadisplay.dot(desc)
    graph, = pydot.graph_from_dot_data(dot_data)
    _LOGGER.info("Writing schema to %r...", schema_file)
    graph.write_png(schema_file)


if __name__ == "__main__":
    cli()
